/*
 * Created on 19 avr. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.bonita;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.w3c.dom.Document;

import hero.interfaces.*;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSession;


/**
 * The aim of this class is to transform data from a Bonita project to a xml file
 * @author sempereb
 */
public class BonitaProjectAdapter {




	/**
	 * Default class constructor 
	 */
	public BonitaProjectAdapter(){};
	
	
	

		
	/**
	 * Get the bonita properties as key value hash map
	 * @param theProjectName : The project concerned.
	 * @return HashMap the table containing the properties.
	 */
	private HashMap getHBonitaData(String theProjectName){
				
		//Hash map where 'Bonita key - value' will be put
		HashMap hMap = new HashMap();
			
		//The local home
		ProjectSessionHome projecth = null;
		
		//The bean
		ProjectSession project  = null;
		
		try{
			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();
			
			//Project creation
			project = projecth.create();
					  
			//Project initialisation
			project.initProject(theProjectName);
			
			//We are now able to acess Bonita api and get properties
			Collection collectionProperties = project.getProperties();
			
			//Number of bean instances responding
			System.out.println( new String
				(new Integer(collectionProperties.size()).toString()) + " Beans retrieved");
			
			//Property value of Bonita
			BnProjectPropertyValue pv = null;
			
			//The current key of the property value object
			String currentKey = "";
			
			//The current value of the property value object
			String currentValue = "";

			//For all the keys we have to retrieve the associated value
			Iterator it = collectionProperties.iterator();
			
			System.out.println();
			System.out.println("***Enterring Bonita API***********");
			System.out.println();
			
			while (it.hasNext()){
				//Getting the values
				pv = (BnProjectPropertyValue)it.next();
			
				//Retrieve the key
				currentKey = pv.getTheKey();
				
				//And the value
				currentValue = pv.getTheValue();
				
				//Adding key and value in a hashMap
				hMap.put(currentKey, currentValue);
				
				//Display
				System.out.println(" Key: " + currentKey);
				System.out.println(" Value: " + currentValue);
				System.out.println();
			}
			
			System.out.println();
			System.out.println("***End of processing Bonita API***");
			System.out.println();

			//Returns the hash map
			return hMap;
		}
		catch(Exception e){
			System.out.println(e + " " + e.getMessage());
			return null;
		}
	}
	
	
	

	
	/**
	 * Provides a Document output for Bonita data
	 * @param theHashMap : The table containing the properties.
	 * @param theProjectName : The project concerned.
	 * @return Document document with the process information : name & properties.
	 * @throws IOExecption
	 */
	private Document generateBonitaDataDocument(HashMap theHashMap, String theProjectName) throws IOException{
		
		//DOM
		Document document = null;
		
		//a parser to generate the xml document from the dataProcess
		DocumentParser parser = new DocumentParser(); 
		
		//Create the dataProcess structure with bonita key value
		DataProject data = new DataProject(theProjectName, theHashMap);
			
		//Generate the xml document representing the dataProcess
		if( data != null )
			document = parser.createDocument(data);	
			
		return document;
	}
	
	
	
	
	/**
	 * Get the Bonita project data identified by its name and put them in a DOM 
	 * @param theProjectName : The name of the project we have to get process informations
	 * @return the document that represents bonita data
	 * @throws LoginException
	 * @throws IOException
	 */
	public Document getProjectData(String theProjectName) throws IOException{
		
		//H map of Bonita data
		HashMap bonitaMap = new HashMap();
		
		//Set Bonita data in H map
		bonitaMap = this.getHBonitaData(theProjectName);
		
		//Document generated
		return this.generateBonitaDataDocument(bonitaMap, theProjectName);
	}
}


