/*
 * Created on 04 mai. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.ggf;

import org.chiba.xml.xforms.ChibaContext;
import org.chiba.xml.xforms.ui.BoundElement;
import org.apache.log4j.Category;

import java.util.Map;
import java.util.HashMap;

/**
 * Stores Servlet application specific properties.
 * @author sempereb
 */
public class WebContextGGF implements ChibaContext {
    
    private static final Category LOGGER = Category.getInstance(ChibaContext.class);
    public static final String REQUEST_PARAMETERS = "chiba.parameters";
    private Map properties=null;
    private GGF ggf = null;

    public WebContextGGF(GGF ggf) {
        this.ggf = ggf;
        this.properties = new HashMap();
    }

    /**
     * Stores a Servlet specific property in the context.
     * @param key the property key
     * @param value the property value
     */
    public void setProperty(String key, Object value) {
        this.properties.put(key,value);
    }

    /**
     * Reads a Servlet specific property from the context.
     * @param key the key for the wanted object
     * @return returns object from context
     */
    public Object getProperty(String key) {
        return this.properties.get(key);
    }

    /** 
     * Return the properties Map
     * @return All context properties in a Map object
     */
    public Map getProperties() {
        return this.properties;
    }

    /**
     * Returns an external name for a given bound control. This can be either a form control or
     * another bound element like label, alert, filename, mediatype....
     *
     * Note: this solution is a bit of a hack. the method would feel better in ServletAdapter itself but
     * this would make necessary another interface to access this method from inside the processor.
     *
     * @param control the bound control.
     * @return an external name for a given bound control.
     */
    public String getExternalName(BoundElement control) {
        String name;

        if (control instanceof org.chiba.xml.xforms.ui.Trigger) {
            name = ggf.getUniqueParameterName(ggf.getTriggerPrefix());
        } else {
            name = ggf.getUniqueParameterName(ggf.getDataPrefix());
        }

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("mapped " + control + " to '" + name + "' ...");
        }

        ((Map)getProperty(WebContextGGF.REQUEST_PARAMETERS)).put(name,control.getId());

        return name;
    }
}