package hero.entity;

import hero.interfaces.BnAgentLightValue;
import hero.interfaces.BnAgentPK;
import hero.interfaces.BnAgentValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnUserLocal;
import hero.interfaces.InvalidValueException;
import hero.util.BonitaServiceLocator;
import hero.util.BonitaServiceException;

import java.util.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;


/**
 * The Entity bean represents a BnUser
 *
 * @author Miguel Valdes
 * @version $Revision: 1.2 $
 *
 * @ejb:bean name="BnAgent"
 *           display-name="BnAgent Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnAgent"
 *           local-jndi-name="ejb/hero/BnAgent_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true" 
 *
 *
 * @ejb:ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 *
 * @ejb.pk method-intf="both"
 * 
 * @jonas.bean
 *     ejb-name="BnAgent"
 *     jndi-name="ejb/hero/BnAgent"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnAgent"
 * @jonas.cleanup create
 * 
 * 
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(a) FROM BnAgent a"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnAgent findByName(java.lang.String pName, int projectId )" view-type="both" result-type-mapping="Remote" 
 *     query="SELECT OBJECT(a) FROM BnAgent a WHERE a.name = ?1 AND a.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnAgentLocal findByName(java.lang.String pName, int projectId )" view-type="both" result-type-mapping="Local" 
 *     query="SELECT OBJECT(a) FROM BnAgent a WHERE a.name = ?1 AND a.bnProject.id = ?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnAgent"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnAgentLight"
 *
 * @jboss:table-name table-name="BnAgent"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 **/

public abstract class BnAgentBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the BnAgent's id.
   *
   * @return Returns an int representing the id of this BnAgent.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="id"
   * @ejb:interface-method view-type="local"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the BnAgent's id.
   *
   * @param pId The id of this BnAgent. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( int pId );
   
   /**
   * Retrieve the BnAgent's state.
   *
   * @return Returns an int representing the state of this BnAgent.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="state"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getState();
   
   /**
   * Set the BnAgent's state.
   *
   * @param pState The state of this BnAgent. Is set at creation time.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setState( int pState );

   /**
   * Retrieve the BnAgent's type.
   *
   * @return Returns the type of this BnAgent.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="type"
   **/
    //   public abstract AgentType getType();
   
   /**
   * Set the BnAgent's type.
   *
   * @param type The type of this BnAgent. Is set at creation time.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
    //   public abstract void setType( AgentType type );

   /**
   * Retrieve the BnAgent's Name.
   *
   * @return Returns an int representing the Name of this BnAgent.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();
   
   /**
   * Set the BnAgent's Name.
   *
   * @param pName The Name of this BnAgent.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String pName );

   /**
   * Retrieve the BnAgent's Description.
   *
   * @return Returns the description of the BnAgent
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="description"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getDescription();
   
   /**
   * Set the BnAgent's Description.
   *
   * @param pDescription The Description of this BnAgent.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setDescription( String pDescription );

    /** Retrieve the BnAgent's Creator.
     *
     * @return Returns a String representing the Name of the BnAgent Creator.
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="AgentCreator" role-name="created-by"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"

     * @ejb:value-object match="*" 
     *       relation="external"
     *       aggregate="hero.interfaces.BnUserLightValue"
     *       aggregate-name="Creator"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnUserLocal getCreator();
    
    /**
     * Set the BnAgent's Creator.
     *
     * @param pCreator The creator of this BnAgent.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setCreator( BnUserLocal pCreator );

    /**
     * @return  the project of the BnAgent
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Agents" role-name="is-agent-of" cascade-delete="yes"
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk= "true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract BnProjectLocal getBnProject();
	
   /**
   * Set the BnProject BnAgent.
   *
   * @param pProject The Name of this BnAgent.  Is set at creation time.
   * @ejb:transaction type="Required"
   * 
   **/
    public abstract void setBnProject(BnProjectLocal pProject);	

    /**
     * @return  the outgoing edges of the agent
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="InAgent" role-name="has-edges-in" target-multiple="yes"
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getOutEdges();

   /**
   * Set the Outgoing edges of the BnAgent
   *
   * @param pEdges the outgoing edges of the BnAgent
   * @ejb:transaction type="Required"
   **/
    public abstract void setOutEdges(Collection pEdges);    

    /**
     * @return Return the creation date of the BnAgent
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Date getCreationDate();

    /**
     * @param pDate the creation date of the BnAgent
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setCreationDate(Date pDate);

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnAgentValue getBnAgentValue();
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnAgentValue(BnAgentValue v);

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnAgentLightValue getBnAgentLightValue();

   /** 
     * @return true if the BnAgent is active
     * @ejb.interface-method view-type="local"
     */ 
    public boolean isActive() { 
	return (this.getState()==hero.interfaces.Constants.Ag.ACTIVE); 
    } 
  
   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a BnAgent based on the supplied BnAgent Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnAgentPK ejbCreate(BnAgentValue nd )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       setBnAgentValue(nd);
		try {
			BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
			this.setId(serviceLocator.generateUniqueId("BnAgent"));
		
		} catch (BonitaServiceException e) {
			throw new CreateException(e.getMessage());
		}
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
   * Create a BnAgent based on the supplied Name and type.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnAgentPK ejbCreate(String name)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setName(name);
       this.setState(hero.interfaces.Constants.Ag.INACTIVE);
       
		try {
			BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
			this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.AGENT));
		
		} catch (BonitaServiceException e) {
			throw new CreateException(e.getMessage());
		}
            
       // This is only possible in CMPs. Otherwise return a valid PK.
       return null;
   }

   
   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
