package hero.entity;
import hero.interfaces.Constants;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnEdgeLocal;
import hero.util.HeroException;

import java.util.*;

public class OrEdgeState extends EdgeState {

	private static OrEdgeState nd;

	public OrEdgeState() {
	}

	public static OrEdgeState getInstance() {
		if (nd == null) {
			nd = new OrEdgeState();
		}
		return nd;
	}

	public int state(BnNodeLocal node) {
		Collection edges = node.getInBnEdges();
		
		//	Instances consistency (may be some activities are not yet created in the instance)
		try {
		String nodeName = node.getName();
		if (isInstance(node.getName()))
			nodeName = getModel(nodeName);
			
		Collection edgesModel = node.getBnProject().getModel().getBnNode(nodeName).getInBnEdges();
		if (edges.size() != edgesModel.size())
			edges = concatEdges(edges, edgesModel);
		}catch (HeroException e){e.printStackTrace();}
		
		Iterator i = edges.iterator();
		int result = Constants.Nd.EDGEINITIAL;
		boolean active = false;
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();

			if (edge.getState() == Constants.Ed.ACTIVE) {
				result = Constants.Nd.ACTIVE;
				active = true;
				break;
			}
			if (edge.getState() == Constants.Ed.INITIAL) {
				result = Constants.Nd.EDGEINITIAL;
				active = true;
			}
			if ((edge.getState() == Constants.Ed.ANTICIPATING)) {
				result = Constants.Nd.ANTACTIVE;
				active = true;
				break;	
			}
			if ((edge.getState() == Constants.Ed.DEAD) && !active)
				result = Constants.Nd.CANCEL;
		}
		return result;

	}
	
	private String getModel(String instanceName) {
		int i = instanceName.indexOf("_instance");
		return (instanceName.substring(0, i));
	}

	private boolean isInstance(String name) {
		return (name.matches(".*_instance.*"));
	}
	
	private Collection concatEdges(Collection edges, Collection edgesModel) {
		Collection result = new ArrayList();
		Iterator ed = edgesModel.iterator();
		while (ed.hasNext())
		{
			boolean found = false;
			BnEdgeLocal el = (BnEdgeLocal)ed.next();
			String nodeName = el.getInBnNode().getName();
			Iterator instEdges = edges.iterator();
			while(instEdges.hasNext())
			{
				BnEdgeLocal insEd = (BnEdgeLocal)instEdges.next();
				String node = insEd.getInBnNode().getName();
				if (this.isInstance(node))
					node = this.getModel(node);
				if (nodeName.equals(node))
				{
					result.add(insEd);
					found = true;
					break;
				}
			}
			if (!found)
				result.add(el);
		}
		return result;
	}

}
