/*
* 02/01/2002 - 15:24:07
*
* NotificationBean.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package hero.message;

// MDB imports
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.util.EventConstants;
import hero.util.HeroException;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;

import hero.util.MailNotification;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;


/**
 * This is a Message Driven Bean
 *
 * @ejb:message-driven
 * @ejb:bean name="NotificationBean"
 *           transaction-type="Container"
 *           acknowledge-mode="Auto-acknowledge"
 *           destination-type="javax.jms.Topic"
 *           subscription-durability="NonDurable"
 *
 * @jboss:destination-jndi-name name="testTopic"
 * @ejb.security-identity run-as="SuperAdmin"
 * @ejb.permission unchecked="yes"   
 * 
 * 
 * @jonas.message-driven-destination jndi-name="testTopic"
 * @jonas.bean  ejb-name="NotificationBean"    
 *              jndi-name="testTopic"
 * @ejb.transaction type= "Required"
 *
 * @author $Author: mvaldes $
 * @version $Revision: 1.3 $
 */

public class NotificationBean implements MessageDrivenBean, MessageListener{  

    //private MessageDrivenContext ctx = null;
    private transient MessageDrivenContext ctx;
	private MailNotification mailNotif = new MailNotification();
   
    private void sendJabber(String toAccount, String subject, String message) throws Exception{
	//ObjectName jabber    = new ObjectName("jboss:service=Jabber"); 
	ObjectName jabber    = new ObjectName("jonas:type=service,name=Jabber");
	List list = MBeanServerFactory.findMBeanServer(null);
	MBeanServer mServer = (MBeanServer)list.iterator().next();
	Object id = (Object) mServer.invoke( jabber,
						 "sendMessage",
						 new Object[] { 
						     toAccount, //JID 
						     null  ,    //Resource
						     subject,   //Subject
						     message    //message
						 },
						 new String[] {
						     "".getClass().getName(),
						     "".getClass().getName(), 
						     "".getClass().getName(), 
						     "".getClass().getName()
						 }
					      );
    }


    public void setMessageDrivenContext(MessageDrivenContext ctx) throws EJBException
    {
    	this.ctx = ctx;
    }

    private Collection usersRegistered(String project, String event) throws Exception {
    	try{
    		ProjectSessionLocalHome home=ProjectSessionUtil.getLocalHome();
    		ProjectSessionLocal prj = home.create();
    		prj.initProject(project);
    	
    		return (prj.getUsers());
    	}catch(Exception usersRegistered) {
    		Collection users = new ArrayList();
    		return (users);
    	}
    } 

    public void ejbCreate()
    { 
    }

    public void ejbRemove() 
    { 
	//    	ctx=null;
    }
    
    public void onMessage(Message message)    
    { 
	BnUserLocal mUser;
	try{
	    String event= (String)message.getObjectProperty(EventConstants.EVENT); 
	    String projectName = (String)message.getObjectProperty(EventConstants.PROJECTNAME);   
	    if(event.equalsIgnoreCase(EventConstants.START) || event.equalsIgnoreCase(EventConstants.TERMINATED)){ 
		Collection c=usersRegistered(projectName,event);
		for(Iterator i=c.iterator();i.hasNext();){
		    //BnUserLightValue userLight=(BnUserLightValue)i.next();
		    String userName = (String)i.next();
		    try {
			BnUserLocalHome userhome=hero.interfaces.BnUserUtil.getLocalHome();
			mUser=userhome.findByName(userName);
		    } catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		    } catch (javax.ejb.FinderException fe) {
			throw new HeroException(userName+" does not exists");
		    }
		    Collection props=mUser.getBnProperties();
		    for(Iterator it=props.iterator();it.hasNext();){
			BnUserPropertyLocal pValue= (BnUserPropertyLocal)it.next();
			if((pValue.getTheKey()).equalsIgnoreCase(event+"Mail") && pValue.getTheValue()!= null){
			    /*if(userLight.getEmail()!=null)
				mailNotif.sendMail(userLight.getEmail(),"Event Notification","BnProject:"+projectName+" Event:"+event);*/
				if(mUser.getEmail()!=null)
				mailNotif.sendMail(mUser.getEmail(),"Event Notification","BnProject:"+projectName+" Event:"+event);
			}
			if((pValue.getTheKey()).equalsIgnoreCase(event+"Jabber") && pValue.getTheValue()!= null ){
			    /*String jab=userLight.getJabber();
			    if(jab!=null){				    
				sendJabber(jab,"Event Notification","Event Notification:"+projectName+" Event:"+event);*/
				if(mUser.getJabber()!=null){				    
					sendJabber(mUser.getJabber(),"Event Notification","Event Notification:"+projectName+" Event:"+event);
			    }
			}    
		    }
		}
	    }  
	}catch(Exception e1){
		ctx.setRollbackOnly();
		e1.printStackTrace();}  	
    }
    
}
