package hero.session;

/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: AdminSessionBean.java,v 1.2 2004/08/18 15:21:10 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.StrutsProjectValue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;


/**
 * Session Bean Template
 *
 * @ejb:bean name="AdminSession"
 *           display-name="AdminSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/AdminSession"
 *           local-jndi-name="ejb/hero/AdminSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Supports"
 * @ejb.permission unchecked="yes" 
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="AdminSession"
 *      jndi-name="ejb/hero/AdminSession"
 * 
 *
 **/

public class AdminSessionBean implements SessionBean, EventConstants  {


   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------
    // Utility variable
	private static final Logger log = Logger.getLogger(AdminSessionBean.class);
   
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;
  
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  

   /**
	  * Get all projects
	  *
	  * @ejb.permission unchecked="yes" 
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Supports"
	  *
	  **/        
	   public Collection getProjects() throws HeroException {
	   log.info("start");
	   BnProjectLocalHome pHome;
	   ArrayList projects = new ArrayList();

	   try {
		   pHome=hero.interfaces.BnProjectUtil.getLocalHome();
	   } catch (javax.naming.NamingException be) {
		   throw new HeroException(be.getMessage());
	   }
	   try {
		   Collection allProjects= pHome.findAll();
		   Iterator i = allProjects.iterator();
		   while(i.hasNext()) {
		   BnProjectLocal pvalue = (BnProjectLocal)i.next();
		   StrutsProjectValue spv = new StrutsProjectValue();
		   spv.setName(pvalue.getName());
		   spv.setCreator(pvalue.getCreator());
		   spv.setState(hero.interfaces.Constants.Pj.projectStateName[pvalue.getState()]);
		   spv.setNact(pvalue.getBnNodes().size());
		   spv.setNusers(pvalue.getBnUsers().size());
		   projects.add(spv);
		   }
		   log.info("End");
		   return((Collection)projects);

	   } catch (FinderException fe) {
		   log.error(fe.getMessage());
		   throw new HeroException(fe.getMessage());}
	   }
   /**
   * Create the AdminSession Bean
   *
   * @throws CreateException 
   *
   * @ejb.permission unchecked="yes" 
   * @ejb:create-method view-type="both"
   **/

    public void ejbCreate() throws CreateException {
	log.debug("ejbCreate");
    }
    
    public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
    }
    
    public void ejbRemove() {
    }
    
    public void ejbActivate() {
    }
    
    public void ejbPassivate() {
    }
    
}
