package hero.session;
/*
*
* ProjectSoapBasicBean.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.util.EventConstants;
import hero.util.HeroException;

import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;

import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.naming.Context;

import java.util.Vector;

import org.apache.log4j.Category;

/**
 * Session Bean Template
 *
 * @ejb:bean name="ProjectSoapBasic"
 *           display-name="ProjectSoapBasic Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/ProjectSoapBasic"
 *
 * @ejb:ejb-ref	ejb-name="ProjectSoapBasic"
 *              	view-type="remote"
 *              	ref-name="ProjectSoapBasic"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user,SuperAdmin"
 * @jonas.bean
 *      ejb-name="ProjectSoapBasic"
 *      jndi-name="ejb/hero/ProjectSoapBasic"
 *
 **/

public class ProjectSoapBasicBean implements SessionBean, EventConstants  {

   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------
    // Utility variable
    private static final Category log = Category.getInstance(ProjectSessionBean.class);
   
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;
   private BnProjectLocalHome pHome;
   private BnProjectLocal mProject;
   private JMSServicesSessionLocal jms;
   private Collection subProjects = new ArrayList();
   private ArrayList visited = new ArrayList();   
   
   
   // -------------------------------------------------------------------------
   // Basic Get Methods
   // -------------------------------------------------------------------------
  
   /**
   * Get ProjectName
   * @return String 
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public String getName() throws HeroException {
	log.debug("getName:");
	return mProject.getName();
    }

   /**
   * Get creator
   * @return String 
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public String getCreator() throws HeroException {
	log.debug("getCreator:");
	return mProject.getCreator();
    }
   
   /**
   * Get the node executor
   *
   * @param name of the node
   * @return The user executor
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/    
    public String getNodeExecutor(String name) throws HeroException {
	log.debug("setNodeDescription: name="+name);
	BnNodeLocal nd=mProject.getBnNode(name);
	BnUserLocal ul = nd.getExecutor();
	return(ul.getName());
    }

    /**
     * Get Node State
     *
     * @param name of the node
     * @return The state of the node
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/
    public int getNodeState(String name) throws HeroException {
	log.debug("setNodeState: name=" + name);
	BnNodeLocal nd = mProject.getBnNode(name);
	int s = nd.getState();
	return (s);
    }

   /**
   * Get all users of the project
   * @return Object[] 
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Object[] getUsers() throws HeroException {
	log.debug("getUsers:");
	Collection users = mProject.getBnUsers();
	ArrayList result=new ArrayList();
	for (Iterator i=users.iterator();i.hasNext();) {
	    result.add(((BnUserLocal)i.next()).getName());
	}
	return result.toArray();
    }

   /**
   * Return all registered users
   * @return Object[] 
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Object[] getAllUsers() throws HeroException {
	log.debug("getAllUsers:");
	BnUserLocalHome uHome;
	Collection users;
	try {
	    uHome = (BnUserLocalHome)hero.interfaces.BnUserUtil.getLocalHome();
	} catch (javax.naming.NamingException be) {
	    throw new HeroException(be.getMessage());
	}	
	try {
	    users=uHome.findAll();
	} catch (FinderException f) {
	    throw new HeroException("FinderException in findByAll users");
	}
	ArrayList result=new ArrayList();
	for (Iterator i=users.iterator();i.hasNext();) {
	    result.add(((BnUserLocal)i.next()).getName());
	}
	return result.toArray();
    }

   /**
   * Get all role names of the project
   * @return Object[]
   *
   * @ejb:interface-method view-type="both
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Object[] getRolesNames() throws HeroException {
	log.debug("getRolesNames:");
	Collection roles = mProject.getBnRoles();
	ArrayList result=new ArrayList();
	for (Iterator i=roles.iterator();i.hasNext();) {
	    result.add(((BnRoleLocal)i.next()).getName());
	}
	return result.toArray();
    }

    /**
     * Get all node names of the project
     *
     * @return Object[]
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       
    public Object[] getNodesNames() throws HeroException {
	log.debug("getNodesNames: ");

	Vector names = new Vector();
	Collection nodes = mProject.getBnNodes();
	Iterator i = nodes.iterator();
	while (i.hasNext())
	{
	    BnNodeLocal n = (BnNodeLocal)i.next();
	    names.add(n.getName());
	}
	return((Collection)names).toArray();
    }

    /**
     * Get all node  of the project
     * @return Object[]
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       
    public Object[] getNodes() throws HeroException {
	log.debug("getNodes: ");
	Vector nodesDetails = new Vector();
	Collection nodes = mProject.getBnNodes();
	
	Iterator i = nodes.iterator();
	while (i.hasNext())
	{
	    BnNodeLocal n = (BnNodeLocal)i.next();
	    String[] details = new String[5];
	    details[0]=n.getName();
	    details[1]=hero.interfaces.Constants.Nd.nodeStateName[n.getState()];
	    details[2]=((BnRoleLocal)n.getBnRole()).getName();
	    if (n.getDeadline()!=null)
		details[3]=n.getDeadline().toString();
	    else
		details[3]="null";
	    details[4]=(new Boolean(n.getAnticipable())).toString();
	    nodesDetails.add(details);
	}
	return((Collection)nodesDetails).toArray();
    }

    /**
     * Get all node information
     * @return Object[]
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       

    public Object[] getNodeDetails(String node) throws HeroException {

	BnNodeLocal nd=mProject.getBnNode(node);
	Vector detail = new Vector();

	detail.add(nd.getName()); 
	detail.add(mProject.getName());
	detail.add(((BnUserLocal)nd.getCreator()).getName());
	detail.add(hero.interfaces.Constants.Nd.nodeStateName[nd.getState()]);
       	detail.add(((BnRoleLocal)nd.getBnRole()).getName());
	if ((BnUserLocal)nd.getExecutor()!=null)
	    detail.add(((BnUserLocal)nd.getExecutor()).getName());
	else
	    detail.add("null");

	if (nd.getDeadline()!=null)	
	    detail.add(nd.getDeadline().toString());
	else detail.add("null");

    	detail.add(new Boolean((nd.getAnticipable())).toString());

	if (nd.getDescription()!=null)
	    detail.add(nd.getDescription());
	else  detail.add("null");
	
	return(detail.toArray());
    }

   /**
   * Get user roles in project
   * @return Object[]
   *
   * @param userName The name of the user
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Object[] getUserRolesInProjectNames(String userName) throws HeroException {
	log.debug("getUserRolesInProject:");
	BnUserLocalHome uHome;
	BnUserLocal user;
	ArrayList result=new ArrayList();
	try {
	    uHome = (BnUserLocalHome)hero.interfaces.BnUserUtil.getLocalHome();
	} catch (javax.naming.NamingException be) {
	    throw new HeroException(be.getMessage());
	}	
	try {
	    user=uHome.findByName(userName);
	} catch (FinderException f) {
	    throw new HeroException("FinderException");
	}
	Collection userRoles = user.getBnRoles();
        Collection projectRoles=mProject.getBnRoles();
	for(Iterator i=userRoles.iterator();i.hasNext();){
	    BnRoleLocal ur=(BnRoleLocal)i.next();		
	    for(Iterator j=projectRoles.iterator();j.hasNext();){	    
		BnRoleLocal up=(BnRoleLocal)j.next();		    
		if(ur.getId()==up.getId())
		    result.add(ur.getName());
	    }
	}
	return result.toArray();
    }

   /**
   * get all in edges of the node
   * @return Object[]
   *
   * @param name the name ot the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/        
    public Object[] getNodeInEdges(String name) throws HeroException {
	log.debug("getNodeInEdges: name="+name);
	
	Vector in=new Vector();
	BnNodeLocal nd=mProject.getBnNode(name);
	
	Collection edgesIn=nd.getInBnEdges();
	Iterator i = edgesIn.iterator();
	while (i.hasNext()){
	    BnEdgeLocal e = (BnEdgeLocal)i.next();
	    String[] details = new String[4];
	    details[0]=e.getName();
	    details[1]=hero.interfaces.Constants.Ed.edgeStateName[e.getState()];
	    BnNodeLocal nIn = (BnNodeLocal)e.getInBnNode();
	    details[2]=nIn.getName();
	    BnNodeLocal nOut = (BnNodeLocal)e.getOutBnNode();
	    details[3]=nOut.getName();
	    in.add(details);
	}
	
	return(in.toArray());
    }
     
    /**
     * Get all BnEdge  of the project
     * @return Object[]
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       
    public Object[] getEdges() throws HeroException {
	log.debug("getEdges: ");
	Vector edgesDetails = new Vector();
	Collection edges = mProject.getBnEdges();
	Iterator i = edges.iterator();
	while (i.hasNext())
	{
	    BnEdgeLocal e = (BnEdgeLocal)i.next();
	    String[] details = new String[4];
	    details[0]=e.getName();
	    details[1]=hero.interfaces.Constants.Ed.edgeStateName[e.getState()];
	    BnNodeLocal nIn = (BnNodeLocal)e.getInBnNode();
	    details[2]=nIn.getName();
	    BnNodeLocal nOut = (BnNodeLocal)e.getOutBnNode();
	    details[3]=nOut.getName();
	    edgesDetails.add(details);
	}
	return(edgesDetails.toArray());
    }
	    
   /**
   * get all out edges of the node
   * @return Object[]
   *
   * @param name the name ot the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/        
    public Object[] getNodeOutEdges(String name) throws HeroException {
	log.debug("getNodeOutEdges: name="+name);

	Vector out=new Vector();
	BnNodeLocal nd=mProject.getBnNode(name);

	Collection edgesOut=nd.getOutBnEdges();
	Iterator i = edgesOut.iterator();
	while (i.hasNext()){
	    BnEdgeLocal e = (BnEdgeLocal)i.next();
	    String[] details = new String[4];
	    details[0]=e.getName();
	    details[1]=hero.interfaces.Constants.Ed.edgeStateName[e.getState()];
	    BnNodeLocal nIn = (BnNodeLocal)e.getInBnNode();
	    details[2]=nIn.getName();
	    BnNodeLocal nOut = (BnNodeLocal)e.getOutBnNode();
	    details[3]=nOut.getName();
	    out.add(details);
	}
	
	return(out.toArray());
    }

	/**
	 * Get the type of the node
	 *
	 * @param name of the node
	 * @return The type of the node
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public int getNodeType(String name) throws HeroException {
		log.debug("setBnNodeType: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getType());
	}
	
	// -------------------------------------------------------------------------
	// ejbCreate and initProject business methods
	// -------------------------------------------------------------------------

	/**
	 * Init the Project Session Bean. You have to call this method after "create" call. After that
	 * all methods of ProjectSession API are available.
	 *
	 * @param projectName project name 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public void initProject(String projectName) throws HeroException {
		Context lContext;
		log.debug("initProject: project=" + projectName);
		  try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			this.mProject = pHome.findByName(projectName);
		  } catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		  } catch (javax.ejb.FinderException fe) {
			 throw new HeroException(fe.getMessage());
		   }
	}

	
	/**
	 * Creates the Project Session Bean. This method is the first one to invoke in order to
	 * use ProjectSession API. If the user is not authorized this method throws an exception.
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void ejbPostCreate(String oldProject, String newProject)
		throws CreateException, RemoteException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		try{
		  JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
		  this.jms=home.create();
		}catch(Exception e){throw new EJBException(e.getMessage());}
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbRemove() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbActivate() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbPassivate() {
	}
}
