package hero.session;
/*
* 1Z/01/2004 - 11:58:09
*
* UserSoapXML.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserValue;
import hero.interfaces.ProjectSessionLocal;
import hero.util.HeroException;
import hero.util.ProjectNodes;
import hero.util.Projects;
import hero.util.UserProperties;
import hero.util.EventConstants;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;


import org.apache.log4j.Category;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;

/**
 * UserSessionXML API based on UserSession API but oriented to web services 
 * calls from a non-Java language. Get methods of this API returns
 * XML.
 *
 * @ejb:bean name="UserSoapXML"
 *           display-name="UserSoapXML Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserSoapXML"
 *
 * @ejb:ejb-ref ejb-name="BnUser"
 *              ref-name="myhero/BnUser"
 * @ejb.permission role-name="Admin,user,SuperAdmin"
 *
 * @jonas.bean
 *      ejb-name="UserSoapXML"
 *      jndi-name="ejb/hero/UserSoapXML"
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 * 
 **/

public class UserSoapXMLBean implements SessionBean, EventConstants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log =
		Category.getInstance(UserSessionBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	// -------------------------------------------------------------------------
	// XML Get Methods (Castor Data Binding)
	// -------------------------------------------------------------------------

	/**
			* Get user project list. Workflow processes associated to this user
			* @return a Collection of BnProjectLightValue objects - the projects list of the user
			* @throws HeroException
			* @ejb:interface-method view-type="both"
			* @ejb:transaction type="Supports"
			*
			**/
	public String getProjectList() throws HeroException {
		Collection pjts = new ArrayList();
		Projects pb = new Projects();
		Collection pva;
		try {
			BnUserLocalHome userhome =
				hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =
				userhome.findByName(mContext.getCallerPrincipal().getName());
			log.debug("getProjectList");
			Collection projects = null;
			log.debug("before getprojects mUser " + mUser);
			projects = mUser.getBnProjects();
			BnUserValue uv = mUser.getBnUserValue();
			log.debug("BnUser Value=" + uv);
			hero.interfaces.BnProjectLightValue[] pv = uv.getBnProjects();
			log.debug("BnProject Value " + pv);
			pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
			log.debug("Projects " + pva);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
				mContext.getCallerPrincipal().getName() + " does not exists");
		}

		try {
			pb.setProject(pva);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(pb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
		* Obtain all user activities from specific project (ready and anticipable state)
		* @param projectName the name of the project
		* @return a String Collection - the list of todo activities ot the user for a project
		* @throws HeroException
		* @ejb:interface-method view-type="both"
		* @ejb:transaction type="Supports"
		*
		**/
	public String getToDoList(String projectName) throws HeroException {
		log.debug("getToDoList projectName=" + projectName);
		ProjectNodes nb = new ProjectNodes();
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection ready;
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
				"BnProject " + projectName + " does not exists");
		}
		log.debug("getToDoList -end projectName=" + projectName);
		try {
			ready =
				nodehome.findUserReady(
					project.getId(),
					mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			log.debug(
				"Empty Todo List for BnUser "
					+ mContext.getCallerPrincipal().getName());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			log.debug("NODE NAME TODO " + nd.getName());
			result.add(nd.getBnNodeLightValue());
		}
		try {
			nb.setNodes(result.toArray());
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Obtain all user activities from specific project (executing and anticipating state)
	 *
	 * @param projectName the name of the project
	 * @return a String Collectio - the list of active activities ot the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getActivityList(String projectName) throws HeroException {
		ProjectNodes nb = new ProjectNodes();
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection executing;
		log.debug("getActivityList -begin projectName=" + projectName);
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
				"BnProject " + projectName + " does not exists");
		}
		try {
			executing =
				nodehome.findUserExecuting(
					project.getId(),
					mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			log.debug(
				"Empty Activity List for BnUser "
					+ mContext.getCallerPrincipal().getName());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			log.debug("NODE NAME Executing " + nd.getName());
			result.add(nd.getBnNodeLightValue());
		}
		try {
			nb.setNodes(result.toArray());
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
		 * Get the user name
		 * 
		 * @return a String - get user name
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 **/
	public String getUser() {
		return mContext.getCallerPrincipal().getName();
	}

	/**
	 * Get the user light value
	 * 
	 * @return a BnUserLightValue object - get user information
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserDetails() throws HeroException {
		try {
			BnUserLocalHome userhome =
				hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =
				userhome.findByName(mContext.getCallerPrincipal().getName());
			try {
				StringWriter sw = new StringWriter();
				Marshaller.marshal(mUser.getBnUserLightValue(), sw);
				return (sw.toString());
			} catch (MarshalException ex1) {
				throw new HeroException(ex1.getMessage());
			} catch (ValidationException ex2) {
				throw new HeroException(ex2.getMessage());
			}
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
				mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}

	/**
	 * Get the user password
	 *
	 * @return a String - get user password
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserPassword() throws HeroException {
		try {
			BnUserLocalHome userhome =
				hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =
				userhome.findByName(mContext.getCallerPrincipal().getName());
			return mUser.getPassword();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
				mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}

	/**
		  * Get User properties
		  *
		  * @return a BnUserPropertyValue Collection - list of user properties
		  * @ejb:interface-method view-type="both"
		  * @ejb:transaction type="Supports"
		  *
		  **/
	public String getUserProperties() throws HeroException {
		Collection prp = new ArrayList();
		BnUserPropertyLocal pl;
		UserProperties ub = new UserProperties();
		try {
			BnUserLocalHome userhome =
				hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =
				userhome.findByName(mContext.getCallerPrincipal().getName());
			Collection pls = mUser.getBnProperties();

			for (Iterator i = pls.iterator(); i.hasNext();) {
				pl = (BnUserPropertyLocal) i.next();
				prp.add(pl.getBnUserPropertyValue());
			}

			try {
				ub.setProperty(prp);
				StringWriter sw = new StringWriter();
				Marshaller.marshal(ub, sw);
				return (sw.toString());
			} catch (MarshalException ex1) {
				throw new HeroException(ex1.getMessage());
			} catch (ValidationException ex2) {
				throw new HeroException(ex2.getMessage());
			}

		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
				mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}

	/**
	* Create the BnUser Session Bean
	*
	* @throws CreateException 
	*
	* @ejb:create-method view-type="both"
	**/
	public void ejbCreate() throws CreateException {
	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}

	public void ejbRemove() {
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

	// utility methods
	private boolean checkRights(ProjectSessionLocal ps) throws HeroException {
		// check that current user is a member of the project
		return ps.containsUser(mContext.getCallerPrincipal().getName());
	}
}


